from collections.abc import Iterator
from typing import Any

def drop_vcs_in_scheme(url: str) -> str: ...
def unsplit_vcs_url(repo_url: str, branch: str | None, subpath: str | None) -> str: ...
def probe_gitlab_host(hostname: str) -> bool: ...
def is_gitlab_site(hostname: str, net_access: bool | None = None) -> bool: ...
def guess_repo_from_url(url: str, net_access: bool | None = None) -> str | None: ...
def probe_gitlabb_host(hostname: str) -> bool: ...
def find_public_repo_url(url: str, net_access: bool | None = None) -> str | None: ...
def browse_url_from_repo_url(
    url: str,
    branch: str | None = None,
    subpath: str | None = None,
    net_access: bool | None = None,
) -> str | None: ...
def plausible_vcs_url(url: str) -> bool: ...
def plausible_vcs_browse_url(url: str) -> bool: ...
def probe_upstream_branch_url(url: str, version: str | None = None) -> bool | None: ...
def canonical_git_repo_url(url: str, net_access: bool | None = None) -> str: ...
def check_repository_url_canonical(url: str, version: str | None = None) -> str: ...
def known_bad_guess(datum: UpstreamDatum) -> bool: ...
def url_from_svn_co_command(command: bytes) -> str | None: ...
def url_from_git_clone_command(command: bytes) -> str | None: ...
def url_from_fossil_clone_command(command: bytes) -> str | None: ...
def url_from_cvs_co_command(command: bytes) -> str | None: ...
def url_from_vcs_command(command: bytes) -> str | None: ...
def find_forge(url: str, net_access: bool | None = None) -> Forge | None: ...
def repo_url_from_merge_request_url(
    url: str, net_access: bool | None = None
) -> str | None: ...
def bug_database_from_issue_url(
    url: str, net_access: bool | None = None
) -> str | None: ...
def guess_bug_database_url_from_repo_url(
    url: str, net_access: bool | None = None
) -> str | None: ...
def bug_database_url_from_bug_submit_url(
    url: str, net_access: bool | None = None
) -> str | None: ...
def bug_submit_url_from_bug_database_url(
    url: str, net_access: bool | None = None
) -> str | None: ...
def check_bug_database_canonical(url: str, net_access: bool | None = None) -> str: ...
def check_bug_submit_url_canonical(url: str, net_access: bool | None = None) -> str: ...
def check_url_canonical(url: str) -> str: ...
def fixup_rcp_style_git_repo_url(url: str) -> str: ...
def check_upstream_metadata(data: UpstreamMetadata) -> None: ...

class ParseError(Exception): ...
class NoSuchForgeProject(Exception): ...

class Forge:
    @classmethod
    def extend_metadata(
        cls, upstream_metadata: UpstreamMetadata, project: str, certainty: str
    ) -> None: ...

    repository_browse_can_be_homepage: bool

class GitHub(Forge): ...
class GitLab(Forge): ...
class SourceForge(Forge): ...
class Launchpad(Forge): ...

SECURE_SCHEMES: list[str]
KNOWN_GITLAB_SITES: list[str]

def find_secure_repo_url(
    url: str, branch: str | None = None, net_access: bool | None = None
) -> str | None: ...
def sanitize_url(url: str) -> str: ...
def convert_cvs_list_to_str(cvs_list: list[str]) -> str: ...
def fixup_broken_git_details(
    url: str, branch: str | None = None, subpath: str | None = None
) -> tuple[str, str | None, str | None]: ...
def guess_upstream_info(
    path: str, trust_package: bool = False
) -> list[UpstreamDatum]: ...
def get_upstream_info(
    path: str,
    trust_package: bool = False,
    net_access: bool | None = None,
    consult_external_directory: bool = True,
    check: bool = True,
) -> UpstreamMetadata: ...
def extend_upstream_metadata(
    upstream_metadata: UpstreamMetadata,
    path: str,
    minimum_certainty: str,
    net_access: bool | None = None,
    consult_external_directory: bool = True,
) -> None: ...
def guess_upstream_metadata(
    path: str,
    trust_package: bool | None = None,
    net_access: bool | None = None,
    consult_external_directory: bool | None = None,
    check: bool | None = None,
) -> UpstreamMetadata: ...
def fix_upstream_metadata(upstream_metadata: UpstreamMetadata) -> None: ...
def guess_upstream_metadata_items(
    path: str, trust_package: bool | None = None, minimum_certainty: str | None = None
) -> Iterator[UpstreamDatum]: ...
def update_from_guesses(
    upstream_metadata: UpstreamMetadata,
    items: list[UpstreamDatum],
) -> list[UpstreamDatum]: ...

class UpstreamMetadata:
    def __init__(self, **kwargs): ...
    ...

    @classmethod
    def from_dict(cls, data, default_certainty: str | None = None): ...

class UpstreamDatum:
    def __init__(self, name, value: Any, certainty: str | None = None): ...  # noqa: ANN401

__version__: str



#!/usr/bin/python3
# Copyright (C) 2018 Jelmer Vernooij <jelmer@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

__all__ = [
    "find_public_repo_url",
    "find_secure_repo_url",
    "convert_cvs_list_to_str",
    "fixup_broken_git_details",
]

from ._upstream_ontologist import (  # noqa: F401
    canonical_git_repo_url,
    convert_cvs_list_to_str,
    drop_vcs_in_scheme,
    find_public_repo_url,
    find_secure_repo_url,
    fixup_broken_git_details,
    fixup_rcp_style_git_repo_url,
)



#!/usr/bin/python3
# Copyright (C) 2018 Jelmer Vernooij <jelmer@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Functions for working with upstream metadata.

This gathers information about upstreams from various places.
Each bit of information gathered is wrapped in a UpstreamDatum
object, which contains the field name.

The fields used here match those in https://wiki.debian.org/UpstreamMetadata

Supported fields:
- Homepage
- Name
- Contact
- Repository
- Repository-Browse
- Bug-Database
- Bug-Submit
- Screenshots
- Archive
- Security-Contact

Extensions for upstream-ontologist.
- SourceForge-Project: Name of the SourceForge project
- Wiki: URL to a wiki
- Summary: A one-line description
- Description: Multi-line description
- License: Short description of the license
- Copyright
- Maintainer
- Authors

Supported, but currently not set.
- FAQ
- Donation
- Documentation
- Registration
- Webservice
"""

from dataclasses import dataclass
from email.utils import parseaddr
from typing import Optional

import ruamel.yaml

from . import _upstream_ontologist

SUPPORTED_CERTAINTIES = ["certain", "confident", "likely", "possible", None]

version_string = "0.2.2"

USER_AGENT = "upstream-ontologist/" + version_string
# Too aggressive?
DEFAULT_URLLIB_TIMEOUT = 3


yaml = ruamel.yaml.YAML(typ="safe")


@dataclass
@yaml.register_class
class Person:
    yaml_tag = "!Person"

    name: str
    email: Optional[str] = None
    url: Optional[str] = None

    def __init__(self, name, email=None, url=None):
        self.name = name
        self.email = email
        if url and url.startswith("mailto:"):
            self.email = url[len("mailto:") :]
            self.url = None
        else:
            self.url = url

    @classmethod
    def from_yaml(cls, constructor, node):
        d = {}
        for k, v in node.value:
            d[k.value] = v.value
        return cls(name=d.get("name"), email=d.get("email"), url=d.get("url"))

    @classmethod
    def from_string(cls, text):
        text = text.replace(" at ", "@")
        text = text.replace(" -at- ", "@")
        text = text.replace(" -dot- ", ".")
        text = text.replace("[AT]", "@")
        if "(" in text and text.endswith(")"):
            (p1, p2) = text[:-1].split("(", 1)
            if p2.startswith("https://") or p2.startswith("http://"):
                url = p2
                if "<" in p1:
                    (name, email) = parseaddr(p1)
                    return cls(name=name, email=email, url=url)
                return cls(name=p1, url=url)
            elif "@" in p2:
                return cls(name=p1, email=p2)
            return cls(text)
        elif "<" in text:
            (name, email) = parseaddr(text)
            return cls(name=name, email=email)
        else:
            return cls(name=text)

    def __str__(self):
        if self.email:
            return f"{self.name} <{self.email}>"
        return self.name


UpstreamDatum = _upstream_ontologist.UpstreamDatum
UpstreamMetadata = _upstream_ontologist.UpstreamMetadata


# If we're setting them new, put Name and Contact first
def upstream_metadata_sort_key(x):
    (k, v) = x
    return {
        "Name": "00-Name",
        "Contact": "01-Contact",
    }.get(k, k)


class UrlUnverifiable(Exception):
    """Unable to check specified URL."""

    def __init__(self, url, reason):
        self.url = url
        self.reason = reason


class InvalidUrl(Exception):
    """Specified URL is invalid."""

    def __init__(self, url, reason):
        self.url = url
        self.reason = reason



#!/usr/bin/python3
# Copyright (C) 2024 Jelmer Vernooij <jelmer@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA


from typing import Any
from unittest import TestCase

from upstream_ontologist import UpstreamMetadata


class UpstreamMetadataFromDictTests(TestCase):
    def test_from_dict(self):
        d = {
            "Name": "foo",
            "Version": "1.2.3",
            "Homepage": "https://example.com",
        }
        metadata = UpstreamMetadata.from_dict(d)
        self.assertEqual(metadata["Name"].value, "foo")
        self.assertEqual(metadata["Version"].value, "1.2.3")
        self.assertEqual(metadata["Homepage"].value, "https://example.com")

    def test_from_dict_missing(self):
        d = {
            "Name": "foo",
            "Version": "1.2.3",
        }
        metadata = UpstreamMetadata.from_dict(d)
        self.assertEqual(metadata["Name"].value, "foo")
        self.assertEqual(metadata["Version"].value, "1.2.3")
        self.assertRaises(KeyError, metadata.__getitem__, "Homepage")

    def test_from_dict_empty(self):
        d: dict[str, Any] = {}
        metadata = UpstreamMetadata.from_dict(d)
        self.assertRaises(KeyError, metadata.__getitem__, "Name")
        self.assertRaises(KeyError, metadata.__getitem__, "Version")
        self.assertRaises(KeyError, metadata.__getitem__, "Homepage")

    def test_from_dict_invalid(self):
        d = {
            "Name": "foo",
            "Version": "1.2.3",
            "Homepage": 42,
        }
        with self.assertRaises(TypeError):
            UpstreamMetadata.from_dict(d)

    def test_from_dict_yaml(self):
        from ruamel.yaml import YAML

        yaml = YAML()
        d = yaml.load("""Name: foo
Version: 1.2.3  # comment
Homepage: https://example.com
""")
        metadata = UpstreamMetadata.from_dict(d)
        self.assertEqual(metadata["Name"].value, "foo")
        self.assertEqual(metadata["Version"].value, "1.2.3")
        self.assertEqual(metadata["Homepage"].value, "https://example.com")

    def test_from_dict_registry(self):
        d = {
            "Name": "foo",
            "Version": "1.2.3",
            "Registry": [{"Name": "conda:conda-forge", "Entry": "r-tsne"}],
        }
        metadata = UpstreamMetadata.from_dict(d)
        self.assertEqual(
            metadata["Registry"].value,
            [{"Name": "conda:conda-forge", "Entry": "r-tsne"}],
        )



#!/usr/bin/python3
# Copyright (C) 2018 Jelmer Vernooij <jelmer@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA


from unittest import TestCase

from upstream_ontologist.vcs import (
    canonical_git_repo_url,
    find_public_repo_url,
    fixup_rcp_style_git_repo_url,
)


class CanonicalizeVcsUrlTests(TestCase):
    def test_github(self):
        self.assertEqual(
            "https://github.com/jelmer/example.git",
            canonical_git_repo_url("https://github.com/jelmer/example"),
        )

    def test_salsa(self):
        self.assertEqual(
            "https://salsa.debian.org/jelmer/example.git",
            canonical_git_repo_url("https://salsa.debian.org/jelmer/example"),
        )
        self.assertEqual(
            "https://salsa.debian.org/jelmer/example.git",
            canonical_git_repo_url("https://salsa.debian.org/jelmer/example.git"),
        )


class FindPublicVcsUrlTests(TestCase):
    def test_github(self):
        self.assertEqual(
            "https://github.com/jelmer/example",
            find_public_repo_url("ssh://git@github.com/jelmer/example"),
        )
        self.assertEqual(
            "https://github.com/jelmer/example",
            find_public_repo_url("https://github.com/jelmer/example"),
        )
        self.assertEqual(
            "https://github.com/jelmer/example",
            find_public_repo_url("git@github.com:jelmer/example"),
        )

    def test_salsa(self):
        self.assertEqual(
            "https://salsa.debian.org/jelmer/example",
            find_public_repo_url("ssh://salsa.debian.org/jelmer/example"),
        )
        self.assertEqual(
            "https://salsa.debian.org/jelmer/example",
            find_public_repo_url("https://salsa.debian.org/jelmer/example"),
        )


class FixupRcpStyleUrlTests(TestCase):
    def test_fixup(self):
        try:
            import breezy  # noqa: F401
        except ModuleNotFoundError:
            self.skipTest("breezy is not available")
        self.assertEqual(
            "ssh://github.com/jelmer/example",
            fixup_rcp_style_git_repo_url("github.com:jelmer/example"),
        )
        self.assertEqual(
            "ssh://git@github.com/jelmer/example",
            fixup_rcp_style_git_repo_url("git@github.com:jelmer/example"),
        )

    def test_leave(self):
        try:
            import breezy  # noqa: F401
        except ModuleNotFoundError:
            self.skipTest("breezy is not available")
        self.assertEqual(
            "https://salsa.debian.org/jelmer/example",
            fixup_rcp_style_git_repo_url("https://salsa.debian.org/jelmer/example"),
        )
        self.assertEqual(
            "ssh://git@salsa.debian.org/jelmer/example",
            fixup_rcp_style_git_repo_url("ssh://git@salsa.debian.org/jelmer/example"),
        )



#!/usr/bin/python3
import sys

from setuptools import setup
from setuptools_rust import Binding, RustExtension

extra_features = []

if sys.platform != "win32":
    extra_features.append("debcargo")

setup(
    rust_extensions=[
        RustExtension(
            "upstream_ontologist._upstream_ontologist",
            "Cargo.toml",
            binding=Binding.PyO3,
            features=["extension-module"] + extra_features,
        ),
    ],
)



#!/usr/bin/python3
# Copyright (C) 2018 Jelmer Vernooij <jelmer@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import logging
from collections.abc import Iterable, Iterator

from . import (
    UpstreamDatum,
    _upstream_ontologist,
)

logger = logging.getLogger(__name__)


def guess_upstream_info(path, trust_package):
    return iter(_upstream_ontologist.guess_upstream_info(path, trust_package))


check_upstream_metadata = _upstream_ontologist.check_upstream_metadata
extend_upstream_metadata = _upstream_ontologist.extend_upstream_metadata
guess_upstream_metadata = _upstream_ontologist.guess_upstream_metadata
known_bad_guess = _upstream_ontologist.known_bad_guess


def filter_bad_guesses(
    guesses: Iterable[UpstreamDatum],
) -> Iterator[UpstreamDatum]:
    return (guess for guess in guesses if not known_bad_guess(guess))


fix_upstream_metadata = _upstream_ontologist.fix_upstream_metadata

guess_upstream_metadata_items = _upstream_ontologist.guess_upstream_metadata_items
update_from_guesses = _upstream_ontologist.update_from_guesses
